using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Data.Common;
using System.Drawing;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace BinarySerialization
{
	public partial class Form1 : Form
	{
		private DataSet ds = null;
		private string connectionString = string.Empty;

		/// <summary>
		/// Constructor
		/// </summary>
		public Form1()
		{
			InitializeComponent();

			// Retrieve the connection string from the configuration file
			ConnectionStringSettings northwind = ConfigurationManager.ConnectionStrings["Northwind"];
			connectionString = northwind.ConnectionString;
		}

		/// <summary>
		/// Retrieves all the orders from the database and shows them in the dataGridView1 grid.
		/// </summary>
		private void button1_Click(object sender, EventArgs e)
		{
			// To use this class, you have to add the System.configuration.dll
			// assembly to the project, and reference the System.Configuration namespace
			
			// To use this data adapter, reference System.Data.SqlClient
			using (SqlDataAdapter da = new SqlDataAdapter("SELECT * FROM Orders", connectionString))
			{
				ds = new DataSet();
				da.Fill(ds, "Orders");
				dataGridView1.DataSource = ds;
				dataGridView1.DataMember = "Orders";
			}
		}

		/// <summary>
		/// Serializes the dataset instance in disk using two different remoting formats
		/// </summary>
		private void button2_Click(object sender, EventArgs e)
		{
			if (ds != null)
			{
				// In the System.Runtime.Serialization.Formatters.Binary namespace
				BinaryFormatter formatter = new BinaryFormatter();
				ds.RemotingFormat = SerializationFormat.Xml;

				// In the System.IO namespace
				using (FileStream stream = new FileStream(@"c:\ADONET_dataset_xml.bin", FileMode.Create))
				{
					formatter.Serialize(stream, ds);
				}

				ds.RemotingFormat = SerializationFormat.Binary;
				using (FileStream binaryStream = new FileStream(@"c:\ADONET_dataset_binary.bin", FileMode.Create))
				{
					formatter.Serialize(binaryStream, ds);
				}
			}
		}

		/// <summary>
		/// Retrieves all the customers from the database, displays them in the grid and
		/// also saves the DataTable instance in disk
		/// </summary>
		private void button3_Click(object sender, EventArgs e)
		{
			DataTable table = new DataTable("Customers");

			using (SqlConnection connection = new SqlConnection(connectionString))
			{
				connection.Open();
				SqlCommand command = new SqlCommand("select * from customers", connection);
				SqlDataReader reader = command.ExecuteReader();
				table.Load(reader);
				dataGridView1.DataSource = table;
				table.WriteXml(@"c:\ADONET_customers_table.xml");
				connection.Close();
			}
		}

		/// <summary>
		/// Retrieves the list of available data providers.
		/// </summary>
		private void button4_Click(object sender, EventArgs e)
		{
			// System.Data.Common namespace...
			DataTable providers = DbProviderFactories.GetFactoryClasses();
			dataGridView1.DataSource = providers;
		}

		/// <summary>
		/// Makes an "agnostic" connection to the database, using the selected provider in the
		/// dataGridView1 datagrid.
		/// </summary>
		private void button5_Click(object sender, EventArgs e)
		{
			DataTable providers = DbProviderFactories.GetFactoryClasses();
			string provider = (string)providers.Rows[dataGridView1.CurrentRow.Index]["InvariantName"];

			DbProviderFactory factory = DbProviderFactories.GetFactory(provider);

			using (DbConnection connection = factory.CreateConnection())
			{
				connection.ConnectionString = connectionString;
				connection.Open();
				DbCommand command = factory.CreateCommand();
				command.CommandText = "SELECT * FROM employees";
				command.Connection = connection;

				DbDataReader reader = command.ExecuteReader();

				DataTable employees = new DataTable("Employees");
				employees.Load(reader);

				dataGridView2.DataSource = employees;
			}
		}

		/// <summary>
		/// Shows the usage of the UpdateBatchSize property in the DbDataAdapter class
		/// </summary>
		private void button6_Click(object sender, EventArgs e)
		{
			if (ds != null)
			{
				using (SqlDataAdapter da = new SqlDataAdapter("SELECT * FROM Orders", connectionString))
				{
					SqlCommandBuilder builder = new SqlCommandBuilder(da);
					da.UpdateBatchSize = 100;
					da.Update(ds.Tables["Orders"]);
				}
			}
		}
	}
}