using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using System.IO;

namespace FileSystemService
{
	public partial class Service1 : ServiceBase
	{
		private string path = string.Empty;
		private string logFilePath = string.Empty;

		/// <summary>
		/// Constructor
		/// </summary>
		public Service1()
		{
			InitializeComponent();
		}

		/// <summary>
		/// Called when the service is started
		/// </summary>
		protected override void OnStart(string[] args)
		{
			path = ConfigurationManager.AppSettings["PathToWatch"];
			logFilePath = ConfigurationManager.AppSettings["LogFilePath"];

			// Set the properties of the FileSystemWatcher instance
			fileSystemWatcher1.Path = path;
			fileSystemWatcher1.EnableRaisingEvents = true;
			fileSystemWatcher1.NotifyFilter = NotifyFilters.CreationTime | NotifyFilters.FileName | 
				NotifyFilters.Size | NotifyFilters.LastAccess;
			fileSystemWatcher1.Created += new FileSystemEventHandler(fileSystemWatcher1_Created);
			fileSystemWatcher1.Renamed += new RenamedEventHandler(fileSystemWatcher1_Renamed);
			fileSystemWatcher1.Deleted += new FileSystemEventHandler(fileSystemWatcher1_Deleted);
		}

		/// <summary>
		/// Called when the service is stopped
		/// </summary>
		protected override void OnStop()
		{
			fileSystemWatcher1.EnableRaisingEvents = false;
		}

		/// <summary>
		/// Triggered when a file is renamed
		/// </summary>
		void fileSystemWatcher1_Renamed(object sender, RenamedEventArgs e)
		{
			string message = String.Format("File renamed: from {0} to {1}", e.OldName, e.Name);
			Log(message);
		}

		/// <summary>
		/// Triggered when a file is created
		/// </summary>
		void fileSystemWatcher1_Created(object sender, FileSystemEventArgs e)
		{
			string message = String.Format("New file: {0}", e.FullPath);
			Log(message);
		}

		/// <summary>
		/// Triggered when a file is deleted
		/// </summary>
		void fileSystemWatcher1_Deleted(object sender, FileSystemEventArgs e)
		{
			string message = String.Format("File deleted: {0}", e.FullPath);
			Log(message);
		}

		/// <summary>
		/// Logs the specified message.
		/// </summary>
		/// <param name="message">The message to log</param>
		private void Log(object message)
		{
			using (StreamWriter writer = File.AppendText(logFilePath))
			{
				writer.WriteLine(message);
				writer.Flush();
				writer.Close();
			}
		}
	}
}
